# multikit "invoked command" for SDP-specified "image" programs
# It is a stub for an helper program that has a different argument signature
# (otherwise we could have just invoked the helper program directly)
# version: 1999.01.07

# The main routine (invoked command):

proc mk_sdp_image {ourProgId ourInternetAddress ourPortNum ourTTL ourPublicKey args} {
	array set attributeArgs $args
	set medium image

	set sessionName "SDP $medium session"
	catch {set sessionName $attributeArgs(nickname)}

	# protocol specification
	set protocol rtp
	catch {set protocol [string tolower $attributeArgs(sdp_transport)]}
	if {$protocol == "rtp" || $protocol == "rtp/avp"} {
		set rtpFlag "-r"
	} else {
		set rtpFlag ""
	}

	# format specification
	set formats ""
	catch {set formats $attributeArgs(sdp_formats)}
	# I'm not sure how we're supposed to handle more than one format:
	if {[llength $formats] > 1} {
		puts "mk_sdp_$medium warning: multiple formats \"$formats\" for session \"$sessionName\""
	}
	set format [lindex $formats 0]
	
	# misc attributes
	set sdp_attributes ""
	catch {set sdp_attributes $attributeArgs(sdp_session_attributes)}
	catch {set sdp_attributes [concat $sdp_attributes $attributeArgs(sdp_media_attributes)]}

	# Set platform-specific parameters:
	global tcl_platform
	if {$tcl_platform(platform) == "windows"} {
	    set toolName winMdp
	    set tmpDirName C:\\\\Temp
	    set addressReuseEnabled 1
	} else { # unix
	    set toolName tkMdp
	    set tmpDirName /tmp
	    set addressReuseEnabled 0 ;# tkMdp doesn't set REUSEADDR
	}
	set execCmd "$toolName -D $tmpDirName -C [list $sessionName] -A $ourInternetAddress/$ourPortNum -t $ourTTL"
	if {![catch {eval exec $execCmd &} pid]} {
	    if {$addressReuseEnabled} {
		noteSpawnedProcess $ourInternetAddress $ourPortNum $ourTTL $pid
	    }
	    return
	}

	tk_messageBox -title "mk_sdp_$medium: error" \
		-message "Failed to exec \"$execCmd\": $pid" \
		-icon warning -type ok
}

proc noteSpawnedProcess {addr port ttl pid} {
	# Now that we've successfully started a background process to implement
	# the invoked command, we inform multikit of the group/port(s) that
	# we're using (in case they need to be tunneled):
        set groupsock [groupsock_create $addr $port $ttl]
	groupsock_turnOnBackgroundReadHandling $groupsock

        # (Note that - unlike RTP/RTCP applications - only one port is used)

	# Finally, periodically check for the death of the spawned process:
	checkSpawnedProcess $pid $groupsock
}

proc checkSpawnedProcess {pid groupsocks} {
	if {![processExists $pid]} {
		foreach groupsock $groupsocks {
			groupsock_delete $groupsock
		}
	} else {
		after 10000 [list checkSpawnedProcess $pid $groupsocks]
	}
}
